import 'dart:async';
import 'dart:math';
import 'package:audio_session/audio_session.dart';
import 'package:cross_file/cross_file.dart';
import 'package:flutter/foundation.dart';
import 'package:get/get.dart';
import 'package:flutter/material.dart';
import 'package:flutter_sound/flutter_sound.dart';
import 'package:lottie/lottie.dart';
import 'package:_private_chat/_private_chat.dart';
import 'package:_private_core/_private_core.dart';
import 'package:permission_handler/permission_handler.dart';

class AudioRecordResult {
  XFile file;
  String ext;
  String fileName;
  int duration;
  AudioRecordResult({
    required this.ext,
    required this.fileName,
    required this.duration,
    required this.file,
  });
}

class WidgetAudioRecord extends StatefulWidget {
  const WidgetAudioRecord({Key? key}) : super(key: key);

  static void playAudioUrl(url, {whenFinished}) async {
    FlutterSoundPlayer player = FlutterSoundPlayer()..openPlayer();
    player.startPlayer(
        fromURI: url,
        whenFinished: () {
          player.logger.d('Play finished');
          player.closePlayer();
          whenFinished?.call();
        });
  }

  @override
  _WidgetAudioRecordState createState() => _WidgetAudioRecordState();
}

class _WidgetAudioRecordState extends State<WidgetAudioRecord>
    with TickerProviderStateMixin {
  FlutterSoundRecorder recorderModule = FlutterSoundRecorder();

  bool _mRecorderIsInited = false;
  AudioSession? session;
  Codec _codec = Codec.aacMP4;
  String _ext = 'aac';
  String get _fileName =>
      'recording_${DateTime.now().millisecondsSinceEpoch}.$_ext';

  @override
  void initState() {
    _openTheRecorder();
    super.initState();
  }

  int nextInt(int min, int max) => min + Random().nextInt((max + 1) - min);

  void setStateIfMounted(f) {
    if (mounted) setState(f);
  }

  @override
  void dispose() {
    session?.setActive(false);
    recorderModule.closeRecorder();
    stopWatchStream();
    super.dispose();
  }

  Future<void> _openTheRecorder() async {
    if (!kIsWeb) {
      var s = await Permission.microphone.status;
      if (s.isGranted || s.isLimited) {
        setState(() {
          isAllowed = true;
        });
      } else {
        if (isAllowed) {
          setState(() {
            isAllowed = false;
          });
        }
      }
    } else {
      setState(() {
        isAllowed = true;
      });
    }
    if (!_mRecorderIsInited) {
      session = await AudioSession.instance;
      await session!.configure(
        const AudioSessionConfiguration(
          avAudioSessionCategory: AVAudioSessionCategory.playAndRecord,
        ),
      );
      session!.setActive(true);
      await recorderModule.openRecorder();
      _mRecorderIsInited = true;
    }
    record();
  }

  Timer? timerr;
  void record() async {
    setStateIfMounted(() {
      recordertime = '00:00';
      secondsStr = '00';
      hoursStr = '00';
      minutesStr = '00';
    });
    if (!await recorderModule.isEncoderSupported(_codec) && kIsWeb) {
      _codec = Codec.opusWebM;
      _ext = 'webm';
      if (!await recorderModule.isEncoderSupported(_codec) && kIsWeb) {
        _mRecorderIsInited = true;
        return;
      }
    }
    //Change Bitrate here
    recorderModule
        .startRecorder(codec: _codec, toFile: _fileName, bitRate: 32000)
        .then((value) {
      setStateIfMounted(() {
        status = 'recording';
      });
      startTimerNow();
    });
  }

  XFile? recordedfile;
  void stopRecorder([callback]) async {
    await recorderModule.stopRecorder().then((value) async {
      setStateIfMounted(() {
        status = 'recorded';
      });
      setStateIfMounted(() {
        recordedfile = XFile(value!);
        recordertime = "$minutesStr:$secondsStr";
      });

      setStateIfMounted(() {
        streamController!.done;
        streamController!.close();
        timerSubscription!.cancel();
      });
      callback?.call();
    });
  }

  void pauseRecorder() async {
    await recorderModule.pauseRecorder().then((value) async {
      setStateIfMounted(() {
        status = 'notrecording';
      });
      pauseTimerNow();
    });
  }

  void resumeRecorder() async {
    await recorderModule.resumeRecorder().then((value) async {
      setStateIfMounted(() {
        status = 'recording';
      });
      resumeTimerNow();
    });
  }

  // ----------------------------- UI --------------------------------
  String status = 'notrecording';

  bool isLoading = false;
  bool isAllowed = false;

  void _requestPermission() async {
    arckipelHaptic();
    PermissionStatus s = await Permission.microphone.request();
    if (s.isRestricted || s.isPermanentlyDenied || s.isDenied) {
      await openAppSettings();
    } else {
      _openTheRecorder();
    }
  }

  Color get _element2 => hexColor('#18191C');
  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        if (status == 'recording') return false;
        return true;
      },
      child: Container(
        height: Get.height * .65,
        decoration: BoxDecoration(
          color: _element2,
          borderRadius: const BorderRadius.vertical(top: Radius.circular(16)),
        ),
        padding: const EdgeInsets.fromLTRB(0, 12, 0, 0),
        child: isLoading == true
            ? const Center(
                child: CircularProgressIndicator(
                    valueColor: AlwaysStoppedAnimation<Color>(Colors.red)))
            : Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  Container(
                    width: 50,
                    height: 8,
                    decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(4),
                        color: hexColor('#151617')),
                  ),
                  Expanded(
                    child: !isAllowed
                        ? _buildWaitingPermission()
                        : Column(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              Lottie.asset(
                                assetlottie('audioRecord'),
                                controller: status != 'recording'
                                    ? AnimationController(vsync: this)
                                    : null,
                                package: '_private_chat',
                                width: Get.width,
                              ),
                              const SizedBox(),
                              Row(
                                mainAxisSize: MainAxisSize.min,
                                children: [
                                  CircleAvatar(
                                    radius: 5,
                                    backgroundColor: hexColor('#F02269')
                                        .withOpacity(status != 'recording'
                                            ? 1
                                            : secondsInt % 2 == 0
                                                ? 1
                                                : 0),
                                  ),
                                  const SizedBox(
                                    width: 8,
                                  ),
                                  Text(
                                    recorderModule.isRecording ||
                                            recorderModule.isPaused
                                        ? "$minutesStr:$secondsStr / 15:00"
                                        : recordertime,
                                    style: w500TextStyle().copyWith(
                                        fontSize: 12, color: Colors.white),
                                  ),
                                  const SizedBox(
                                    width: 16,
                                  ),
                                ],
                              ),
                              const SizedBox(),
                              Padding(
                                padding:
                                    const EdgeInsets.symmetric(horizontal: 24),
                                child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceAround,
                                  children: [
                                    GestureDetector(
                                      onTap: () {
                                        arckipelHaptic();
                                        Navigator.pop(context);
                                      },
                                      child: const CircleAvatar(
                                        backgroundColor: Colors.black,
                                        radius: 28,
                                        child: RotatedBox(
                                          quarterTurns: 135,
                                          child: Icon(Icons.close_rounded,
                                              color: Colors.white, size: 26),
                                        ),
                                      ),
                                    ),
                                    GestureDetector(
                                      onTap: () {
                                        arckipelHaptic();
                                        if (!_mRecorderIsInited) {
                                          return;
                                        }
                                        recorderModule.isPaused
                                            ? resumeRecorder()
                                            : pauseRecorder();
                                      },
                                      child: CircleAvatar(
                                        backgroundColor: Colors.black,
                                        radius: 28,
                                        child: Icon(
                                            recorderModule.isPaused
                                                ? Icons.play_arrow_rounded
                                                : Icons.pause,
                                            color: Colors.white,
                                            size: 26),
                                      ),
                                    ),
                                    GestureDetector(
                                      onTap: () {
                                        arckipelHaptic();
                                        if (!recorderModule.isStopped) {
                                          stopRecorder(() {
                                            Navigator.pop(
                                                context,
                                                recordedfile != null
                                                    ? AudioRecordResult(
                                                        fileName: _fileName,
                                                        ext: _ext,
                                                        file: recordedfile!,
                                                        duration: secondsInt)
                                                    : null);
                                          });
                                        } else {
                                          Navigator.pop(
                                              context,
                                              recordedfile != null
                                                  ? AudioRecordResult(
                                                      fileName: _fileName,
                                                      ext: _ext,
                                                      file: recordedfile!,
                                                      duration: secondsInt)
                                                  : null);
                                        }
                                      },
                                      child: const CircleAvatar(
                                        backgroundColor: Colors.black,
                                        radius: 28,
                                        child: RotatedBox(
                                          quarterTurns: 135,
                                          child: Icon(Icons.send,
                                              color: Colors.white, size: 24),
                                        ),
                                      ),
                                    ),
                                  ],
                                ),
                              ),
                              const SizedBox(),
                              const SizedBox(),
                            ],
                          ),
                  ),
                ],
              ),
      ),
    );
  }

  Widget _buildWaitingPermission() {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Text(
          'youmustallowaccessmicrophone'.tr,
          style: w500TextStyle().copyWith(color: Colors.white, fontSize: 18),
          textAlign: TextAlign.center,
        ),
        const SizedBox(
          height: 12,
        ),
        Center(
          child: InkWell(
            onTap: _requestPermission,
            child: Container(
              decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.circular(8),
              ),
              padding: const EdgeInsets.symmetric(vertical: 12, horizontal: 24),
              child: Text(
                'requestpermission'.tr,
                style: w500TextStyle().copyWith(color: Colors.black),
                textAlign: TextAlign.center,
              ),
            ),
          ),
        )
      ],
    );
  }

  //------ Timer Widget Section Below:
  bool flag = true;
  Stream<int>? timerStream;
  // ignore: cancel_subscriptions
  StreamSubscription<int>? timerSubscription;
  int secondsInt = 0;
  String hoursStr = '00';
  String minutesStr = '00';
  String secondsStr = '00';
  String recordertime = '00:00';
  // ignore: close_sinks
  StreamController<int>? streamController;
  Stream<int> stopWatchStream([int start = 0]) {
    Timer? timer;
    Duration timerInterval = const Duration(seconds: 1);
    int counter = start;

    void stopTimer() {
      if (timer != null) {
        timer!.cancel();
        timer = null;
        counter = 0;
        streamController!.close();
      }
    }

    void tick(_) {
      counter++;
      streamController!.add(counter);
      if (!flag) {
        stopTimer();
      }
    }

    void startTimer() {
      timer = Timer.periodic(timerInterval, tick);
    }

    streamController = StreamController<int>(
      onListen: startTimer,
      onCancel: stopTimer,
      onResume: startTimer,
      onPause: stopTimer,
    );

    return streamController!.stream;
  }

  startTimerNow() {
    timerStream = stopWatchStream();
    timerSubscription = timerStream!.listen((int newTick) {
      setStateIfMounted(() {
        hoursStr =
            ((newTick / (60 * 60)) % 60).floor().toString().padLeft(2, '0');
        minutesStr = ((newTick / 60) % 60).floor().toString().padLeft(2, '0');
        secondsStr = (newTick % 60).floor().toString().padLeft(2, '0');
        secondsInt = newTick;
      });
      if (secondsInt == 60 * 15) stopRecorder();
      appDebugPrint('secondsInt: $secondsInt');
    });
  }

  pauseTimerNow() {
    timerSubscription!.cancel();
  }

  resumeTimerNow() {
    timerStream = stopWatchStream(secondsInt);
    timerSubscription = timerStream!.listen((int newTick) {
      setStateIfMounted(() {
        hoursStr =
            ((newTick / (60 * 60)) % 60).floor().toString().padLeft(2, '0');
        minutesStr = ((newTick / 60) % 60).floor().toString().padLeft(2, '0');
        secondsStr = (newTick % 60).floor().toString().padLeft(2, '0');
        secondsInt = newTick;
      });
      if (secondsInt == 60 * 15) stopRecorder();
      appDebugPrint('secondsInt: $secondsInt');
    });
  }
}

class SoundSpectrum extends CustomPainter {
  final int percent;
  final Color processColor;
  final Color backGroundColor;

  final List<int> listPreRandom;
  final List<int> listAfterRandom;

  final int ratioWitdTileOnSize;
  final int numberWave;

  const SoundSpectrum({
    required this.listPreRandom,
    required this.listAfterRandom,
    required this.percent,
    required this.processColor,
    required this.backGroundColor,
    this.ratioWitdTileOnSize = 45,
    this.numberWave = 30,
  });

  int nextInt(int min, int max) => min + Random().nextInt((max + 1) - min);

  @override
  void paint(Canvas canvas, Size size) {
    int index = (percent * numberWave / 100).ceil();

    // create the Shader from the gradient and the bounding square
    final double widthTile = size.width / ratioWitdTileOnSize;
    final Paint paint = Paint()
      ..strokeWidth = widthTile
      ..strokeCap = StrokeCap.round
      ..style = PaintingStyle.stroke
      ..color = processColor;

    final Paint paintGrey = Paint()
      ..strokeWidth = widthTile
      ..strokeCap = StrokeCap.round
      ..style = PaintingStyle.stroke
      ..color = backGroundColor;
    double inc = 0;
    for (int i = 0; i < numberWave / 2; i++) {
      final Path path = Path();
      inc += 1.5;
      double x = 1;
      int y = listPreRandom[i];
      path.moveTo(widthTile * inc, -(x) - y);
      path.lineTo(widthTile * inc, (x) + y);
      if (index >= i)
        canvas.drawPath(path, paint);
      else
        canvas.drawPath(path, paintGrey);
    }
    for (int i = (numberWave / 2).ceil(); i > 0; i--) {
      final Path path = Path();
      inc += 1.5;
      double x = 1;
      int y = listAfterRandom[i - 1];
      path.moveTo(widthTile * inc, -(x) - y);
      path.lineTo(widthTile * inc, (x) + y);
      if (index < numberWave / 2)
        canvas.drawPath(path, paintGrey);
      else if ((index - numberWave / 2) >= (i - numberWave / 2).abs())
        canvas.drawPath(path, paint);
      else
        canvas.drawPath(path, paintGrey);
    }
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
