import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:_private_chat/chat_detail/bloc/chat_detail_bloc.dart';
import 'package:_private_chat/chat_detail/chat_helper.dart';
import 'package:_private_chat/chat_list/widget/widget_user_builder.dart';
import 'package:_private_chat/utils/utils.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_core/widgets/widgets.dart';

import '../message.dart';
import 'widget_message_state.dart';

bool get isMobile => !kIsWeb && !Platform.isMacOS && !Platform.isWindows;

class WidgetMessageOwner extends StatelessWidget {
  final Map<String, dynamic> data;
  final String peerId;
  final String toPeerId;
  final dynamic delivered;
  final List<Message> messages;
  const WidgetMessageOwner({
    Key? key,
    required this.messages,
    required this.data,
    required this.peerId,
    required this.toPeerId,
    required this.delivered,
  }) : super(key: key);

  bool get isMe => data[DB_KEY_FROM] == loggedFirebaseId;

  int get timestamp => data[DB_KEY_TIMESTAMP];

  MessageIndexOfGroup indexOfGroup(List<Message> messages, item) {
    if (messages.length == 1) return MessageIndexOfGroup.none;
    if (messages.indexOf(item) == 0) return MessageIndexOfGroup.top;
    if (messages.indexOf(item) == messages.length - 1) {
      return MessageIndexOfGroup.bottom;
    }
    return MessageIndexOfGroup.center;
  }

  @override
  Widget build(BuildContext context) {
    return WidgetUserChatListBuilder(
        id: peerId,
        builder: (fullname, thumbnail) {
          final Map<String, dynamic> seens = SeenProvider.of(context).value;
          bool seen = false;
          List<String> seenUsers = [];
          seens.forEach((key, value) {
            if (key != loggedFirebaseId) {
              seen = value >= timestamp;
              if (value >= timestamp) seenUsers.add(key);
            }
          });
          if (chatOptions!.chatOwnerBubbleBuilder != null) {
            return chatOptions!.chatOwnerBubbleBuilder!(
                _buildState(seen, seenUsers),
                isMe ? appPrefs.loginUser!.thumbnail : thumbnail,
                isMe ? 'you'.tr : fullname,
                messages.map((e) => _buildMessageChild(e)).toList(),
                isMe);
          }
          if (isMobile) {
            return Padding(
              padding: const EdgeInsets.only(top: 16, bottom: 4),
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: isMe
                    ? [
                        Expanded(
                          child: Padding(
                            padding: const EdgeInsets.symmetric(horizontal: 12),
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Padding(
                                  padding:
                                      const EdgeInsets.only(top: 8, bottom: 4),
                                  child: Row(
                                    crossAxisAlignment: CrossAxisAlignment.end,
                                    children: [
                                      const Spacer(),
                                      _buildState(seen, seenUsers),
                                      const SizedBox(
                                        width: 12,
                                      ),
                                      Text(
                                        'you'.tr,
                                        style: w500TextStyle().copyWith(
                                          fontSize: 15,
                                          color: Colors.white,
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                                ...messages
                                    .map((e) => _buildMessageChild(e))
                                    .toList()
                              ],
                            ),
                          ),
                        ),
                        WidgetAvatar.withoutBorder(
                          radius: 20,
                          imageUrl:
                              isMe ? appPrefs.loginUser!.thumbnail : thumbnail,
                        )
                      ]
                    : [
                        WidgetAvatar.withoutBorder(
                          radius: 20,
                          imageUrl:
                              isMe ? appPrefs.loginUser!.thumbnail : thumbnail,
                        ),
                        Expanded(
                          child: Padding(
                            padding: const EdgeInsets.symmetric(horizontal: 12),
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.end,
                              children: [
                                Padding(
                                  padding:
                                      const EdgeInsets.only(top: 8, bottom: 4),
                                  child: Row(
                                    crossAxisAlignment: CrossAxisAlignment.end,
                                    children: [
                                      Text(
                                        fullname,
                                        style: w500TextStyle().copyWith(
                                          fontSize: 15,
                                          color: Colors.white,
                                        ),
                                      ),
                                      const SizedBox(
                                        width: 12,
                                      ),
                                      _buildState(seen, seenUsers),
                                    ],
                                  ),
                                ),
                                ...messages
                                    .map((e) => _buildMessageChild(e))
                                    .toList()
                              ],
                            ),
                          ),
                        ),
                      ],
              ),
            );
          }
          return Padding(
            padding: const EdgeInsets.only(top: 16, bottom: 4),
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                WidgetAvatar.withoutBorder(
                  radius: 20,
                  imageUrl: isMe ? appPrefs.loginUser!.thumbnail : thumbnail,
                ),
                const SizedBox(
                  width: 12,
                ),
                Expanded(
                  child: Column(
                    children: [
                      Padding(
                        padding: const EdgeInsets.only(top: 8, bottom: 4),
                        child: Row(
                          crossAxisAlignment: CrossAxisAlignment.end,
                          children: [
                            Text(
                              isMe ? 'you'.tr : fullname,
                              style: w500TextStyle().copyWith(
                                fontSize: 15,
                                color: Colors.white,
                              ),
                            ),
                            const SizedBox(
                              width: 12,
                            ),
                            _buildState(seen, seenUsers),
                          ],
                        ),
                      ),
                      ...messages.map((e) => _buildMessageChild(e)).toList()
                    ],
                  ),
                )
              ],
            ),
          );
        });
  }

  Widget _buildState(seen, seenUsers) {
    return WidgetMessageState(
      seen: seen,
      seenUsers: seenUsers,
      delivered: delivered,
      timestamp: timestamp,
      isMe: isMe,
      firebaseId: toPeerId,
    );
  }

  Widget _buildMessageChild(e) {
    return Message.wrapMessage(
      snappableKey: e.snappableKey,
      animationController: e.animationController,
      child: WidgetMessageContent(
        delivered: ChatHelper.getMessageStatus(peerId, e.timestamp),
        key: ValueKey(e.timestamp),
        data: e.tempDoc ?? Map.from(e.doc!.data() as Map),
        indexOfGroup: indexOfGroup(messages, e),
      ),
    );
  }
}

class WidgetMessageContent extends StatefulWidget {
  final MessageIndexOfGroup indexOfGroup;
  final Map<String, dynamic> data;
  final dynamic delivered;
  const WidgetMessageContent({
    Key? key,
    required this.indexOfGroup,
    required this.data,
    required this.delivered,
  }) : super(key: key);

  @override
  _WidgetMessageContentState createState() => _WidgetMessageContentState();
}

class _WidgetMessageContentState extends State<WidgetMessageContent> {
  late Map<String, dynamic> data = widget.data;
  late ChatDetailBloc bloc = Get.find<ChatDetailBloc>();

  int get timestamp => data[DB_KEY_TIMESTAMP];

  String get content => ChatHelper.decrypt(
      bloc.conversationData![DB_KEY_ENCRYPT_KEY], data[DB_KEY_CONTENT]);

  bool get isMe => data[DB_KEY_FROM] == loggedFirebaseId;

  bool hasReactions(context) {
    Map<dynamic, dynamic> reactions = ReactionProvider.of(context).value;
    return reactions.containsKey('$timestamp') &&
        reactions['$timestamp'] != null;
  }

  dynamic reactions(context) {
    Map<dynamic, dynamic> reactions = ReactionProvider.of(context).value;
    return reactions['$timestamp'] ?? <dynamic, dynamic>{};
  }

  @override
  Widget build(BuildContext context) {
    Widget child;
    switch (MessageType.values[data[DB_KEY_MESSAGE_TYPE]]) {
      case MessageType.audio:
      case MessageType.image:
      case MessageType.gif:
      case MessageType.lottie:
        child = chatOptions!
            .chatMessageContentBuilder(data, widget.indexOfGroup, content);
        break;
      default:
        child = ValueListenableBuilder<Map>(
          valueListenable: bloc.messageUpdatedValue,
          builder: (_, value, child) {
            if (value.containsKey('$timestamp') &&
                value['$timestamp'] != null) {
              return chatOptions!.chatMessageContentBuilder(
                  data, widget.indexOfGroup, value['$timestamp'] ?? "");
            }
            return child!;
          },
          child: chatOptions!
              .chatMessageContentBuilder(data, widget.indexOfGroup, content),
        );
        break;
    }

    if (hasReactions(context)) {
      Widget reactionWidget = Row(
        children: reactions(context).keys.map<Widget>((e) {
          Widget emoji = chatOptions!.chatReactionIconBuilder(
              chatEmojis
                  .firstWhere((_e) => _e.value == reactions(context)[e])
                  .asset,
              1);

          return Padding(
            padding: isMobile
                ? (isMe
                    ? const EdgeInsets.only(left: 8)
                    : const EdgeInsets.only(right: 8))
                : const EdgeInsets.only(right: 8),
            child: GestureDetector(
              onTap: () {
                if (e == loggedFirebaseId) {
                  Get.find<ChatDetailBloc>()
                      .add(ChatDetailUnsendReactionMessagesEvent(data));
                }
              },
              child: emoji,
            ),
          );
        }).toList(),
      );
      child = chatOptions!.chatReactionWrapBuilder(child, reactionWidget, isMe);
    }

    return chatOptions!.chatMessagePositionBuilder(
        chatOptions!.chatDetailContentActionBuilder(child, data, timestamp),
        isMe);
  }
}

class EmojiMessage {
  final String asset;
  final int value;
  EmojiMessage({required this.asset, required this.value});
}

List<EmojiMessage> chatEmojis = [
  EmojiMessage(asset: 'Angry Face', value: 1),
  EmojiMessage(asset: 'Astonished Face', value: 2),
  EmojiMessage(asset: 'Disappointed Face', value: 3),
  EmojiMessage(asset: 'Loudly Crying Face', value: 4),
  EmojiMessage(asset: 'Rolling on the Floor Laughing', value: 5),
  EmojiMessage(asset: 'Smiling Face with Heart-Eyes', value: 6),
  EmojiMessage(asset: 'Thumbs Down', value: 7),
  EmojiMessage(asset: 'Thumbs Up', value: 8),
];

List<String> chatEmojis2 = [
  "Face with Monocle",
  "Alien",
  "Anger Symbol",
  "Angry Face with Horns",
  "Angry Face",
  "Anxious Face with Sweat",
  "Astonished Face",
  "Backhand Index Pointing Down",
  "Backhand Index Pointing Right",
  "Backhand Index Pointing Up",
  "Beaming Face with Smiling Eyes",
  "Beating Heart",
  "Black Heart",
  "Blue Heart",
  "Bomb",
  "Broken Heart",
  "Call Me Hand",
  "Christmas Tree",
  "Clapping Hands",
  "Clown Face",
  "Cold Face",
  "Compass",
  "Confounded Face",
  "Confused Face",
  "Cowboy Hat Face",
  "Disappointed Face",
  "Dizzy Face",
  "Dizzy",
  "Drooling Face",
  "Explode",
  "Exploding Head",
  "Expressionless Face",
  "Eyes",
  "Face Blowing a Kiss",
  "Face Savoring Food",
  "Face Screaming in Fear",
  "Face Vomiting",
  "Face with Hand Over Mouth",
  "Face with Head-Bandage",
  "Face with Medical Mask",
  "Face with Raised Eyebrow",
  "Face with Rolling Eyes",
  "Hugging Face",
  "Loudly Crying Face",
  "Rolling on the Floor Laughing",
  "Smiling Face with Heart-Eyes",
  "Thumbs Down",
  "Thumbs Up",
];
