import 'dart:async';
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:_private_core/_private_core.dart';
import 'package:photo_view/photo_view.dart';

class PhotoViewScreen extends StatefulWidget {
  final ImageProvider imageProvider;
  const PhotoViewScreen({Key? key, required this.imageProvider})
      : super(key: key);

  @override
  _PhotoViewScreenState createState() => _PhotoViewScreenState();
}

class _PhotoViewScreenState extends State<PhotoViewScreen> {
  @override
  Widget build(BuildContext context) {
    Completer<ui.Image> completer = Completer<ui.Image>();
    widget.imageProvider
        .resolve(const ImageConfiguration())
        .addListener(ImageStreamListener((ImageInfo info, bool _) {
      completer.complete(info.image);
    }));

    return GestureDetector(
      onTap: () => Get.back(),
      child: Scaffold(
        backgroundColor: Colors.transparent,
        body: SafeArea(
          child: Center(
            child: SizedBox(
              width: Get.width - 20,
              child: FutureBuilder<ui.Image>(
                future: completer.future,
                builder: (_, snapshot) {
                  if (!snapshot.hasData) return const SizedBox();
                  return AspectRatio(
                    aspectRatio: snapshot.data!.width / snapshot.data!.height,
                    child: Container(
                      decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(24)),
                      child: Container(
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(16)),
                        child: ClipRRect(
                          borderRadius: BorderRadius.circular(20),
                          child: PhotoView.customChild(
                            minScale: 1.0,
                            child: Container(
                              decoration: BoxDecoration(
                                  image: DecorationImage(
                                      image: widget.imageProvider)),
                            ),
                          ),
                        ),
                      ),
                    ),
                  );
                },
              ),
            ),
          ),
        ),
      ),
    );
  }
}
