import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:_private_core/_private_core.dart';

class WidgetTopNotification extends StatelessWidget {
  final String title;
  final Widget? icon;
  final bool blink;
  final bool? grip;

  const WidgetTopNotification(
      {Key? key,
      this.blink = false,
      required this.title,
      this.icon,
      this.grip = true})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      clipBehavior: Clip.none,
      children: [
        Container(
          width: Get.width,
          margin: EdgeInsets.only(
              top: 5 + MediaQuery.of(context).padding.top, left: 5, right: 5),
          padding:
              const EdgeInsets.only(bottom: 20, top: 24, left: 24, right: 24),
          decoration: BoxDecoration(
              color: bottomSheetBg,
              borderRadius: const BorderRadius.only(
                  bottomLeft: Radius.circular(20),
                  bottomRight: Radius.circular(20),
                  topLeft: Radius.circular(20),
                  topRight: Radius.circular(20))),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              Text(
                title,
                textAlign: TextAlign.center,
                style: w700TextStyle(),
              ),
              if (icon != null) icon!,
            ],
          ),
        ),
        if (grip == true)
          Positioned(
            bottom: -15,
            left: Get.width / 2 - 30,
            child: Container(
              height: 7.5,
              width: 60,
              decoration: BoxDecoration(
                  color: bottomSheetGrip,
                  borderRadius: BorderRadius.circular(99)),
            ),
          )
      ],
    );
  }
}

class WidgetTopNotificationCustom extends StatelessWidget {
  final Widget child;
  final bool? grip;
  final Color? backgroundColor;
  final Color? gripColor;

  const WidgetTopNotificationCustom({
    Key? key,
    required this.child,
    this.gripColor,
    this.backgroundColor,
    this.grip = true,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      clipBehavior: Clip.none,
      children: [
        Container(
          width: Get.width,
          margin: EdgeInsets.only(
              top: 5 + MediaQuery.of(context).padding.top, left: 5, right: 5),
          padding:
              const EdgeInsets.only(bottom: 20, top: 24, left: 24, right: 24),
          decoration: BoxDecoration(
              color: backgroundColor ?? bottomSheetBg,
              borderRadius: const BorderRadius.only(
                  bottomLeft: Radius.circular(12),
                  bottomRight: Radius.circular(12),
                  topLeft: Radius.circular(12),
                  topRight: Radius.circular(12))),
          child: child,
        ),
        if (grip == true)
          Positioned(
            bottom: -15,
            left: Get.width / 2 - 30,
            child: Container(
              height: 7.5,
              width: 60,
              decoration: BoxDecoration(
                  color: gripColor ?? bottomSheetGrip,
                  borderRadius: BorderRadius.circular(99)),
            ),
          )
      ],
    );
  }
}
